@@ -1461,7 +1461,7 @@ list_richcompare(PyObject *v, PyObject *w, int op)
 		PyObject *res;
 		switch (op) {
 		case Py_LT: cmp = vs <  ws; break;
-		case Py_LE: cmp = ws <= ws; break;
+		case Py_LE: cmp = vs <= ws; break;
 		case Py_EQ: cmp = vs == ws; break;
 		case Py_NE: cmp = vs != ws; break;
 		case Py_GT: cmp = vs >  ws; break;
